@@ -35,6 +35,11 @@ fi
 # git clean -fdX removes all ignored (build) files
 commands="\
 git clean -fdX
+cd ./libmariadb
+git submodule update
+cd ../storage/rocksdb/rocksdb
+git submodule update
+cd ../../..
 
 path=`dirname $0`
 . \"$path/autorun.sh\""
