@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /*
   Functions to handle initializating and allocationg of all mysys & debug
@@ -305,7 +305,7 @@ my_bool my_thread_init(void)
                          STACK_DIRECTION * (long)my_thread_stack_size;
 
   mysql_mutex_lock(&THR_LOCK_threads);
-  tmp->id= ++thread_id;
+  tmp->id= tmp->dbug_id= ++thread_id;
   ++THR_thread_count;
   mysql_mutex_unlock(&THR_LOCK_threads);
   tmp->init= 1;
@@ -385,6 +385,7 @@ void my_thread_end(void)
 
     /* Trash variable so that we can detect false accesses to my_thread_var */
     tmp->init= 2;
+    TRASH(tmp, sizeof(*tmp));
     free(tmp);
   }
 }
@@ -410,7 +411,7 @@ my_thread_id my_thread_dbug_id()
     my_thread_init().
   */
   struct st_my_thread_var *tmp= my_thread_var;
-  return tmp ? tmp->id : 0;
+  return tmp ? tmp->dbug_id : 0;
 }
 
 #ifdef DBUG_OFF
