@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /********************************************************************
   memory debugger
@@ -55,6 +55,9 @@ struct st_irem
   struct st_irem *next;        /* Linked list of structures       */
   struct st_irem *prev;        /* Other link                      */
   size_t datasize;             /* Size requested                  */
+#if SIZEOF_SIZE_T == 4
+  size_t pad;                  /* Compensate 32bit datasize */
+#endif
 #ifdef HAVE_BACKTRACE
   void *frame[SF_REMEMBER_FRAMES]; /* call stack                  */
 #endif
@@ -282,8 +285,8 @@ static void warn(const char *format,...)
   va_list args;
   DBUG_PRINT("error", ("%s", format));
   va_start(args,format);
-  vfprintf(stderr, format, args);
   fflush(stderr);
+  vfprintf(stderr, format, args);
   va_end(args);
 
 #ifdef HAVE_BACKTRACE
@@ -375,7 +378,7 @@ void sf_report_leaked_memory(my_thread_id id)
     {
       my_thread_id tid = irem->thread_id && irem->flags & MY_THREAD_SPECIFIC ?
                          irem->thread_id : 0;
-      fprintf(stderr, "Warning: %4lu bytes lost at %p, allocated by T@%lu at ",
+      fprintf(stderr, "Warning: %4lu bytes lost at %p, allocated by T@%llu at ",
               (ulong) irem->datasize, (char*) (irem + 1), tid);
       print_stack(irem->frame);
       total+= irem->datasize;
