@@ -27,23 +27,18 @@ srcdir=""
 
 args=""
 defaults=""
-defaults_group_suffix=""
 mysqld_opt=""
 user=""
+silent_startup="--silent-startup"
 
 force=0
 in_rpm=0
 ip_only=0
 cross_bootstrap=0
-install_params="create database if not exists mysql;
-create database if not exists test;
-use mysql;"
+install_params=""
 auth_root_authentication_method=normal
 auth_root_socket_user='root'
 
-dirname0=`dirname $0 2>/dev/null`
-dirname0=`dirname $dirname0 2>/dev/null`
-
 usage()
 {
   cat <<EOF
@@ -70,9 +65,6 @@ Usage: $0 [OPTIONS]
   --defaults-extra-file=name
                        Read this file after the global files are read.
   --defaults-file=name Only read default options from the given file name.
-  --defaults-group-suffix=name
-                       In addition to the given groups, read also groups with
-                       this suffix
   --force              Causes mysql_install_db to run even if DNS does not
                        work.  In that case, grant table entries that
                        normally use hostnames will use IP addresses.
@@ -150,13 +142,11 @@ parse_arguments()
         # where a chown of datadir won't help)
         user=`parse_arg "$arg"` ;;
       --skip-name-resolve) ip_only=1 ;;
-      --verbose) verbose=1 ;; # Obsolete
+      --verbose) verbose=1 ; silent_startup="" ;;
       --rpm) in_rpm=1 ;;
       --help) usage ;;
       --no-defaults|--defaults-file=*|--defaults-extra-file=*)
         defaults="$arg" ;;
-      --defaults-group-suffix=*)
-        defaults_group_suffix="$arg" ;;
 
       --cross-bootstrap|--windows)
         # Used when building the MariaDB system tables on a different host than
@@ -243,7 +233,7 @@ cannot_find_file()
   echo
   echo "If you compiled from source, you need to either run 'make install' to"
   echo "copy the software into the correct location ready for operation."
-  echo "If you don't want to do a full install, you can use the --srcdir"
+  echo "If you don't want to do a full install, you can use the --srcddir"
   echo "option to only install the mysql database and privilege tables"
   echo
   echo "If you are using a binary release, you must either be at the top"
@@ -286,9 +276,6 @@ then
     cannot_find_file my_print_defaults $basedir/bin $basedir/extra
     exit 1
   fi
-elif test -n "$dirname0" -a -x "$dirname0/@bindir@/my_print_defaults"
-then
-  print_defaults="$dirname0/@bindir@/my_print_defaults"
 else
   print_defaults="@bindir@/my_print_defaults"
 fi
@@ -301,11 +288,9 @@ fi
 
 # Now we can get arguments from the groups [mysqld] and [mysql_install_db]
 # in the my.cfg file, then re-run to merge with command line arguments.
-parse_arguments `"$print_defaults" $defaults $defaults_group_suffix --mysqld mysql_install_db`
+parse_arguments `"$print_defaults" $defaults --mysqld mysql_install_db`
 parse_arguments PICK-ARGS-FROM-ARGV "$@"
 
-rel_mysqld="$dirname0/@INSTALL_SBINDIR@/mysqld"
-
 # Configure paths to support files
 if test -n "$srcdir"
 then
@@ -344,20 +329,11 @@ then
     cannot_find_file fill_help_tables.sql @pkgdata_locations@
     exit 1
   fi
-# relative from where the script was run for a relocatable install
-elif test -n "$dirname0" -a -x "$rel_mysqld" -a ! "$rel_mysqld" -ef "@sbindir@/mysqld"
-then
-  basedir="$dirname0"
-  bindir="$basedir/@INSTALL_BINDIR@"
-  resolveip="$bindir/resolveip"
-  mysqld="$rel_mysqld"
-  srcpkgdatadir="$basedir/@INSTALL_MYSQLSHAREDIR@"
-  buildpkgdatadir="$basedir/@INSTALL_MYSQLSHAREDIR@"
 else
   basedir="@prefix@"
   bindir="@bindir@"
   resolveip="$bindir/resolveip"
-  mysqld="@sbindir@/mysqld"
+  mysqld="@libexecdir@/mysqld"
   srcpkgdatadir="@pkgdatadir@"
   buildpkgdatadir="@pkgdatadir@"
 fi
@@ -433,7 +409,7 @@ then
 fi
 
 # Create database directories
-for dir in "$ldata"
+for dir in "$ldata" "$ldata/mysql" "$ldata/test"
 do
   if test ! -d "$dir"
   then
@@ -476,7 +452,7 @@ fi
 mysqld_bootstrap="${MYSQLD_BOOTSTRAP-$mysqld}"
 mysqld_install_cmd_line()
 {
-  "$mysqld_bootstrap" $defaults $defaults_group_suffix "$mysqld_opt" --bootstrap \
+  "$mysqld_bootstrap" $defaults "$mysqld_opt" --bootstrap $silent_startup\
   "--basedir=$basedir" "--datadir=$ldata" --log-warnings=0 --enforce-storage-engine="" \
   $args --max_allowed_packet=8M \
   --net_buffer_length=16K
@@ -495,7 +471,7 @@ SET @auth_root_socket=NULL;" ;;
 SET @skip_auth_root_nopasswd=1;
 SET @auth_root_socket='$auth_root_socket_user';" ;;
 esac
-if { echo "$install_params"; cat "$create_system_tables" "$create_system_tables2" "$fill_system_tables"; } | eval "$filter_cmd_line" | mysqld_install_cmd_line > /dev/null
+if { echo "use mysql;$install_params"; cat "$create_system_tables" "$create_system_tables2" "$fill_system_tables" "$fill_help_tables" "$maria_add_gis_sp"; } | eval "$filter_cmd_line" | mysqld_install_cmd_line > /dev/null
 then
   s_echo "OK"
 else
@@ -510,7 +486,7 @@ else
   echo
   echo "You can also try to start the mysqld daemon with:"
   echo
-  echo "    shell> $mysqld --skip-grant-tables --general-log &"
+  echo "    shell> $mysqld --skip-grant --general-log &"
   echo
   echo "and use the command line tool $bindir/mysql"
   echo "to connect to the mysql database and look at the grant tables:"
@@ -521,8 +497,8 @@ else
   echo "Try 'mysqld --help' if you have problems with paths.  Using"
   echo "--general-log gives you a log in $ldata that may be helpful."
   link_to_help
-  echo "You can find the latest source at https://downloads.mariadb.org and"
-  echo "the maria-discuss email list at https://launchpad.net/~maria-discuss"
+  echo "MariaDB is hosted on launchpad; You can find the latest source and"
+  echo "email lists at http://launchpad.net/maria"
   echo
   echo "Please check all of the above before submitting a bug report"
   echo "at http://mariadb.org/jira"
@@ -530,27 +506,6 @@ else
   exit 1
 fi
 
-s_echo "Filling help tables..."
-if { echo "use mysql;"; cat "$fill_help_tables"; } | mysqld_install_cmd_line > /dev/null
-then
-  s_echo "OK"
-else
-  echo
-  echo "WARNING: HELP FILES ARE NOT COMPLETELY INSTALLED!"
-  echo "The \"HELP\" command might not work properly."
-fi
-
-s_echo "Creating OpenGIS required SP-s..."
-if { echo "use mysql;"; cat "$maria_add_gis_sp"; } | mysqld_install_cmd_line > /dev/null
-then
-  s_echo "OK"
-else
-  echo
-  echo "WARNING: OPENGIS REQUIRED SP-S WERE NOT COMPLETELY INSTALLED!"
-  echo "GIS extentions might not work properly."
-fi
-
-
 # Don't output verbose information if running inside bootstrap or using
 # --srcdir for testing.  In such cases, there's no end user looking at
 # the screen.
