@@ -103,7 +103,7 @@ public:
   virtual String *val_str_ascii(String *buffer);
 
 protected:
-  virtual bool calc_value(String *arg, String *buffer) = 0;
+  virtual bool calc_value(const String *arg, String *buffer) = 0;
 };
 
 
@@ -132,7 +132,7 @@ public:
   { return get_item_copy<Item_func_inet6_aton>(thd, mem_root, this); }
 
 protected:
-  virtual bool calc_value(String *arg, String *buffer);
+  virtual bool calc_value(const String *arg, String *buffer);
 };
 
 
@@ -166,7 +166,7 @@ public:
   { return get_item_copy<Item_func_inet6_ntoa>(thd, mem_root, this); }
 
 protected:
-  virtual bool calc_value(String *arg, String *buffer);
+  virtual bool calc_value(const String *arg, String *buffer);
 };
 
 
