@@ -57,7 +57,7 @@ static int my_b_encr_read(IO_CACHE *info, uchar *Buffer, size_t Count)
 
   if (info->seek_not_done)
   {
-    size_t wpos;
+    my_off_t wpos;
 
     pos_offset= pos_in_file % info->buffer_length;
     pos_in_file-= pos_offset;
@@ -92,7 +92,7 @@ static int my_b_encr_read(IO_CACHE *info, uchar *Buffer, size_t Count)
       DBUG_RETURN(1);
     }
 
-    elength= wlength - (ebuffer - wbuffer);
+    elength= wlength - (uint)(ebuffer - wbuffer);
     set_iv(iv, pos_in_file, crypt_data->inbuf_counter);
 
     if (encryption_crypt(ebuffer, elength, info->buffer, &length,
@@ -106,7 +106,7 @@ static int my_b_encr_read(IO_CACHE *info, uchar *Buffer, size_t Count)
 
     DBUG_ASSERT(length <= info->buffer_length);
 
-    copied= MY_MIN(Count, length - pos_offset);
+    copied= MY_MIN(Count, (size_t)(length - pos_offset));
 
     memcpy(Buffer, info->buffer + pos_offset, copied);
     Count-= copied;
@@ -120,7 +120,7 @@ static int my_b_encr_read(IO_CACHE *info, uchar *Buffer, size_t Count)
 
     if (wlength < crypt_data->block_length && pos_in_file < info->end_of_file)
     {
-      info->error= pos_in_file - old_pos_in_file;
+      info->error= (int)(pos_in_file - old_pos_in_file);
       DBUG_RETURN(1);
     }
   } while (Count);
@@ -145,10 +145,9 @@ static int my_b_encr_write(IO_CACHE *info, const uchar *Buffer, size_t Count)
 
   if (info->seek_not_done)
   {
-    DBUG_ASSERT(info->pos_in_file % info->buffer_length == 0);
-    size_t wpos= info->pos_in_file / info->buffer_length * crypt_data->block_length;
+    DBUG_ASSERT(info->pos_in_file == 0);
 
-    if ((mysql_file_seek(info->file, wpos, MY_SEEK_SET, MYF(0)) == MY_FILEPOS_ERROR))
+    if ((mysql_file_seek(info->file, 0, MY_SEEK_SET, MYF(0)) == MY_FILEPOS_ERROR))
     {
       info->error= -1;
       DBUG_RETURN(1);
@@ -185,7 +184,7 @@ static int my_b_encr_write(IO_CACHE *info, const uchar *Buffer, size_t Count)
       my_errno= 1;
       DBUG_RETURN(info->error= -1);
     }
-    wlength= elength + ebuffer - wbuffer;
+    wlength= elength + (uint)(ebuffer - wbuffer);
 
     if (length == info->buffer_length)
     {
