@@ -927,6 +927,7 @@ Explain_aggr_filesort::Explain_aggr_filesort(MEM_ROOT *mem_root,
   for (ORDER *ord= filesort->order; ord; ord= ord->next)
   {
     sort_items.push_back(ord->item[0], mem_root);
+    sort_directions.push_back(&ord->direction, mem_root);
   }
   filesort->tracker= &tracker;
 }
@@ -940,10 +941,13 @@ void Explain_aggr_filesort::print_json_members(Json_writer *writer,
   str.length(0);
   
   List_iterator_fast<Item> it(sort_items);
-  Item *item;
+  List_iterator_fast<ORDER::enum_order> it_dir(sort_directions);
+  Item* item;
+  ORDER::enum_order *direction;
   bool first= true;
   while ((item= it++))
   {
+    direction= it_dir++;
     if (first)
       first= false;
     else
@@ -951,6 +955,8 @@ void Explain_aggr_filesort::print_json_members(Json_writer *writer,
       str.append(", ");
     }
     append_item_to_str(&str, item);
+    if (*direction == ORDER::ORDER_DESC)
+      str.append(" desc");
   }
 
   writer->add_member("sort_key").add_str(str.c_ptr_safe());
