@@ -1,7 +1,6 @@
 #ifndef INCLUDES_MYSQL_SQL_LIST_H
 #define INCLUDES_MYSQL_SQL_LIST_H
 /* Copyright (c) 2000, 2012, Oracle and/or its affiliates.
-   Copyright (c) 2019, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #ifdef USE_PRAGMA_INTERFACE
 #pragma interface			/* gcc class implementation */
@@ -23,7 +22,8 @@
 #include "my_sys.h"                    /* alloc_root, TRASH, MY_WME,
                                           MY_FAE, MY_ALLOW_ZERO_PTR */
 #include "m_string.h"                           /* bfill */
-#include "thr_malloc.h"                         /* sql_alloc */
+
+THD *thd_get_current_thd();
 
 /* mysql standard class memory allocator */
 
@@ -32,22 +32,22 @@ class Sql_alloc
 public:
   static void *operator new(size_t size) throw ()
   {
-    return sql_alloc(size);
+    return thd_alloc(thd_get_current_thd(), size);
   }
   static void *operator new[](size_t size) throw ()
   {
-    return sql_alloc(size);
+    return thd_alloc(thd_get_current_thd(), size);
   }
   static void *operator new[](size_t size, MEM_ROOT *mem_root) throw ()
   { return alloc_root(mem_root, size); }
   static void *operator new(size_t size, MEM_ROOT *mem_root) throw ()
   { return alloc_root(mem_root, size); }
-  static void operator delete(void *ptr, size_t size) { TRASH_FREE(ptr, size); }
+  static void operator delete(void *ptr, size_t size) { TRASH(ptr, size); }
   static void operator delete(void *ptr, MEM_ROOT *mem_root)
   { /* never called */ }
   static void operator delete[](void *ptr, MEM_ROOT *mem_root)
   { /* never called */ }
-  static void operator delete[](void *ptr, size_t size) { TRASH_FREE(ptr, size); }
+  static void operator delete[](void *ptr, size_t size) { TRASH(ptr, size); }
 #ifdef HAVE_valgrind
   bool dummy_for_valgrind;
   inline Sql_alloc() :dummy_for_valgrind(0) {}
@@ -86,14 +86,6 @@ public:
     next= elements ? tmp.next : &first;
   }
 
-  SQL_I_List& operator=(const SQL_I_List &tmp)
-  {
-    elements= tmp.elements;
-    first= tmp.first;
-    next= tmp.next;
-    return *this;
-  }
-
   inline void empty()
   {
     elements= 0;
@@ -184,13 +176,6 @@ public:
       first == rhs.first &&
       last == rhs.last;
   }
-  base_list& operator=(const base_list &rhs)
-  {
-    elements= rhs.elements;
-    first= rhs.first;
-    last= elements ? rhs.last : &first;
-    return *this;
-  }
 
   inline void empty() { elements=0; first= &end_of_list; last=&first;}
   inline base_list() { empty(); }
@@ -205,7 +190,9 @@ public:
   */
   inline base_list(const base_list &tmp) :Sql_alloc()
   {
-    *this= tmp;
+    elements= tmp.elements;
+    first= tmp.first;
+    last= elements ? tmp.last : &first;
   }
   /**
     Construct a deep copy of the argument in memory root mem_root.
@@ -214,7 +201,7 @@ public:
     list_copy_and_replace_each_value after creating a copy.
   */
   base_list(const base_list &rhs, MEM_ROOT *mem_root);
-  inline base_list(bool) { }
+  inline base_list(bool error) { }
   inline bool push_back(void *info)
   {
     if (((*last)=new list_node(info, &end_of_list)))
@@ -323,13 +310,10 @@ public:
   */
   inline void swap(base_list &rhs)
   {
-    list_node **rhs_last=rhs.last;
     swap_variables(list_node *, first, rhs.first);
+    swap_variables(list_node **, last, rhs.last);
     swap_variables(uint, elements, rhs.elements);
-    rhs.last= last == &first ? &rhs.first : last;
-    last = rhs_last == &rhs.first ? &first : rhs_last;
   }
-
   inline list_node* last_node() { return *last; }
   inline list_node* first_node() { return first;}
   inline void *head() { return first->info; }
@@ -467,6 +451,11 @@ public:
     el= &current->next;
     return current->info;
   }
+  /* Get what calling next() would return, without moving the iterator */
+  inline void *peek()
+  {
+    return (*el)->info;
+  }
   inline void *next_fast(void)
   {
     list_node *tmp;
@@ -519,6 +508,10 @@ public:
   {
     return el == &list->last_ref()->next;
   }
+  inline bool at_end()
+  {
+    return current == &end_of_list;
+  }
   friend class error_list_iterator;
 };
 
@@ -526,6 +519,7 @@ template <class T> class List :public base_list
 {
 public:
   inline List() :base_list() {}
+  inline List(const List<T> &tmp) :base_list(tmp) {}
   inline List(const List<T> &tmp, MEM_ROOT *mem_root) :
     base_list(tmp, mem_root) {}
   inline bool push_back(T *a) { return base_list::push_back(a); }
@@ -565,6 +559,7 @@ public:
   List_iterator() : base_list_iterator() {}
   inline void init(List<T> &a) { base_list_iterator::init(a); }
   inline T* operator++(int) { return (T*) base_list_iterator::next(); }
+  inline T* peek() { return (T*) base_list_iterator::peek(); }
   inline T *replace(T *a)   { return (T*) base_list_iterator::replace(a); }
   inline T *replace(List<T> &a) { return (T*) base_list_iterator::replace(a); }
   inline void rewind(void)  { base_list_iterator::rewind(); }
@@ -622,7 +617,7 @@ inline void bubble_sort(List<T> *list_to_sort,
     swap= FALSE;
     while ((item2= it++) && (ref2= it.ref()) != last_ref)
     {
-      if (sort_func(item1, item2, arg) < 0)
+      if (sort_func(item1, item2, arg) > 0)
       {
         *ref1= item2;
         *ref2= item1;
@@ -665,6 +660,10 @@ struct ilink
     if (next) next->prev=prev;
     prev=0 ; next=0;
   }
+  inline void assert_linked()
+  {
+    DBUG_ASSERT(prev != 0 && next != 0);
+  }
   virtual ~ilink() { unlink(); }		/*lint -e1740 */
 };
 
