@@ -1,7 +1,7 @@
 /*********** File AM Zip C++ Program Source Code File (.CPP) ***********/
 /* PROGRAM NAME: FILAMZIP                                              */
 /* -------------                                                       */
-/*  Version 1.1                                                        */
+/*  Version 1.3                                                        */
 /*                                                                     */
 /* COPYRIGHT:                                                          */
 /* ----------                                                          */
@@ -45,12 +45,12 @@
 
 #define WRITEBUFFERSIZE (16384)
 
-bool ZipLoadFile(PGLOBAL g, char *zfn, char *fn, char *entry, bool append, bool mul);
+bool ZipLoadFile(PGLOBAL g, PCSZ zfn, PCSZ fn, PCSZ entry, bool append, bool mul);
 
 /***********************************************************************/
 /*  Compress a file in zip when creating a table.                      */
 /***********************************************************************/
-static bool ZipFile(PGLOBAL g, ZIPUTIL *zutp, char *fn, char *entry, char *buf)
+static bool ZipFile(PGLOBAL g, ZIPUTIL *zutp, PCSZ fn, PCSZ entry, char *buf)
 {
 	int   rc = RC_OK, size_read, size_buf = WRITEBUFFERSIZE;
 	FILE *fin;
@@ -88,7 +88,7 @@ static bool ZipFile(PGLOBAL g, ZIPUTIL *zutp, char *fn, char *entry, char *buf)
 /***********************************************************************/
 /*  Find and Compress several files in zip when creating a table.      */
 /***********************************************************************/
-static bool ZipFiles(PGLOBAL g, ZIPUTIL *zutp, char *pat, char *buf)
+static bool ZipFiles(PGLOBAL g, ZIPUTIL *zutp, PCSZ pat, char *buf)
 {
 	char filename[_MAX_PATH];
 	int  rc;
@@ -203,7 +203,7 @@ static bool ZipFiles(PGLOBAL g, ZIPUTIL *zutp, char *pat, char *buf)
 /***********************************************************************/
 /*  Load and Compress a file in zip when creating a table.             */
 /***********************************************************************/
-bool ZipLoadFile(PGLOBAL g, char *zfn, char *fn, char *entry, bool append, bool mul)
+bool ZipLoadFile(PGLOBAL g, PCSZ zfn, PCSZ fn, PCSZ entry, bool append, bool mul)
 {
 	char    *buf;
 	bool     err;
@@ -228,7 +228,7 @@ bool ZipLoadFile(PGLOBAL g, char *zfn, char *fn, char *entry, bool append, bool
 /***********************************************************************/
 /*  Constructors.                                                      */
 /***********************************************************************/
-ZIPUTIL::ZIPUTIL(PSZ tgt)
+ZIPUTIL::ZIPUTIL(PCSZ tgt)
 {
 	zipfile = NULL;
 	target = tgt;
@@ -269,7 +269,7 @@ void ZIPUTIL::getTime(tm_zip& tmZip)
 /*	append:		set true to append the zip file													 */
 /*  return:	true if open, false otherwise.														 */
 /***********************************************************************/
-bool ZIPUTIL::open(PGLOBAL g, char *filename, bool append)
+bool ZIPUTIL::open(PGLOBAL g, PCSZ filename, bool append)
 {
 	if (!zipfile && !(zipfile = zipOpen64(filename,
 		                               append ? APPEND_STATUS_ADDINZIP
@@ -290,12 +290,15 @@ void ZIPUTIL::close()
 		zipfile = NULL;
 	}	// endif zipfile
 
+	if (fp)
+		fp->Count = 0;
+
 }	// end of close
 
 /***********************************************************************/
 /*  OpenTableFile: Open a DOS/UNIX table file from a ZIP file.         */
 /***********************************************************************/
-bool ZIPUTIL::OpenTable(PGLOBAL g, MODE mode, char *fn, bool append)
+bool ZIPUTIL::OpenTable(PGLOBAL g, MODE mode, PCSZ fn, bool append)
 {
 	/*********************************************************************/
 	/*  The file will be compressed.                                     */
@@ -338,10 +341,10 @@ bool ZIPUTIL::OpenTable(PGLOBAL g, MODE mode, char *fn, bool append)
 /***********************************************************************/
 /*  Add target in zip file.				   		      												 */
 /***********************************************************************/
-bool ZIPUTIL::addEntry(PGLOBAL g, char *entry)
+bool ZIPUTIL::addEntry(PGLOBAL g, PCSZ entry)
 {
 	//?? we dont need the stinking time
-	zip_fileinfo zi = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+	zip_fileinfo zi = { {0, 0, 0, 0, 0, 0}, 0, 0, 0 };
 
 	getTime(zi.tmz_date);
 	target = entry;
@@ -382,10 +385,11 @@ void ZIPUTIL::closeEntry()
 /***********************************************************************/
 /*  Constructors.                                                      */
 /***********************************************************************/
-UNZIPUTL::UNZIPUTL(PSZ tgt, bool mul)
+UNZIPUTL::UNZIPUTL(PCSZ tgt, bool mul)
 {
 	zipfile = NULL;
 	target = tgt;
+	pwd = NULL;
 	fp = NULL;
 	memory = NULL;
 	size = 0;
@@ -401,6 +405,26 @@ UNZIPUTL::UNZIPUTL(PSZ tgt, bool mul)
 #endif
 } // end of UNZIPUTL standard constructor
 
+UNZIPUTL::UNZIPUTL(PDOSDEF tdp)
+{
+	zipfile = NULL;
+	target = tdp->GetEntry();
+	pwd = tdp->Pwd;
+	fp = NULL;
+	memory = NULL;
+	size = 0;
+	entryopen = false;
+	multiple = tdp->GetMul();
+	memset(fn, 0, sizeof(fn));
+
+	// Init the case mapping table.
+#if defined(__WIN__)
+	for (int i = 0; i < 256; ++i) mapCaseTable[i] = toupper(i);
+#else
+	for (int i = 0; i < 256; ++i) mapCaseTable[i] = i;
+#endif
+} // end of UNZIPUTL standard constructor
+
 #if 0
 UNZIPUTL::UNZIPUTL(PZIPUTIL zutp)
 {
@@ -418,8 +442,8 @@ UNZIPUTL::UNZIPUTL(PZIPUTIL zutp)
 /* This code is the copyright property of Alessandro Felice Cantatore. */
 /* http://xoomer.virgilio.it/acantato/dev/wildcard/wildmatch.html			 */
 /***********************************************************************/
-bool UNZIPUTL::WildMatch(PSZ pat, PSZ str) {
-	PSZ  s, p;
+bool UNZIPUTL::WildMatch(PCSZ pat, PCSZ str) {
+	PCSZ s, p;
 	bool star = FALSE;
 
 loopStart:
@@ -434,7 +458,7 @@ loopStart:
 			if (!*++pat) return TRUE;
 			goto loopStart;
 		default:
-			if (mapCaseTable[(uchar)*s] != mapCaseTable[(uchar)*p])
+			if (mapCaseTable[(uint)*s] != mapCaseTable[(uint)*p])
 				goto starCheck;
 			break;
 		} /* endswitch */
@@ -453,7 +477,7 @@ starCheck:
 /*  param: filename	path and the filename of the zip file to open.		 */
 /*  return:	true if open, false otherwise.														 */
 /***********************************************************************/
-bool UNZIPUTL::open(PGLOBAL g, char *filename)
+bool UNZIPUTL::open(PGLOBAL g, PCSZ filename)
 {
 	if (!zipfile && !(zipfile = unzOpen64(filename)))
 		sprintf(g->Message, "Zipfile open error on %s", filename);
@@ -472,6 +496,9 @@ void UNZIPUTL::close()
 		zipfile = NULL;
 	}	// endif zipfile
 
+	if (fp)
+		fp->Count = 0;
+
 }	// end of close
 
 /***********************************************************************/
@@ -543,7 +570,7 @@ int UNZIPUTL::nextEntry(PGLOBAL g)
 /***********************************************************************/
 /*  OpenTableFile: Open a DOS/UNIX table file from a ZIP file.         */
 /***********************************************************************/
-bool UNZIPUTL::OpenTable(PGLOBAL g, MODE mode, char *fn)
+bool UNZIPUTL::OpenTable(PGLOBAL g, MODE mode, PCSZ fn)
 {
 	/*********************************************************************/
 	/*  The file will be decompressed into virtual memory.               */
@@ -581,7 +608,7 @@ bool UNZIPUTL::OpenTable(PGLOBAL g, MODE mode, char *fn)
 			if (openEntry(g))
 				return true;
 
-			if (size > 0)	{
+			if (size > 0) {
 				/*******************************************************************/
 				/*  Link a Fblock. This make possible to automatically close it    */
 				/*  in case of error g->jump.                                      */
@@ -612,6 +639,28 @@ bool UNZIPUTL::OpenTable(PGLOBAL g, MODE mode, char *fn)
 	return false;
 } // end of OpenTableFile
 
+/***********************************************************************/
+/*  Insert only if the entry does not exist.   												 */
+/***********************************************************************/
+bool UNZIPUTL::IsInsertOk(PGLOBAL g, PCSZ fn)
+{
+	bool ok = true, b = open(g, fn);
+
+	if (!b) {
+		if (!target || *target == 0) {
+			unz_global_info64 ginfo;
+			int err = unzGetGlobalInfo64(zipfile, &ginfo);
+
+			ok = !(err == UNZ_OK && ginfo.number_entry > 0);
+		} else						// Check if the target exist
+			ok = (unzLocateFile(zipfile, target, 0) != UNZ_OK);
+
+		unzClose(zipfile);
+	} // endif b
+
+	return ok;
+} // end of IsInsertOk
+
 /***********************************************************************/
 /*  Open target in zip file.						      												 */
 /***********************************************************************/
@@ -625,18 +674,24 @@ bool UNZIPUTL::openEntry(PGLOBAL g)
 	if (rc != UNZ_OK) {
 		sprintf(g->Message, "unzGetCurrentFileInfo64 rc=%d", rc);
 		return true;
-	} else if ((rc = unzOpenCurrentFile(zipfile)) != UNZ_OK) {
+	} else if ((rc = unzOpenCurrentFilePassword(zipfile, pwd)) != UNZ_OK) {
 		sprintf(g->Message, "unzOpen fn=%s rc=%d", fn, rc);
 		return true;
 	}	// endif rc
 
 	size = finfo.uncompressed_size;
-	memory = new char[size + 1];
+
+	try {
+		memory = new char[size + 1];
+	} catch (...) {
+		strcpy(g->Message, "Out of memory");
+		return true;
+	} // end try/catch
 
 	if ((rc = unzReadCurrentFile(zipfile, memory, size)) < 0) {
 		sprintf(g->Message, "unzReadCurrentFile rc = %d", rc);
 		unzCloseCurrentFile(zipfile);
-		free(memory);
+		delete[] memory;
 		memory = NULL;
 		entryopen = false;
 	} else {
@@ -661,7 +716,7 @@ void UNZIPUTL::closeEntry()
 	}	// endif entryopen
 
 	if (memory) {
-		free(memory);
+		delete[] memory;
 		memory = NULL;
 	}	// endif memory
 
@@ -675,15 +730,17 @@ void UNZIPUTL::closeEntry()
 UNZFAM::UNZFAM(PDOSDEF tdp) : MAPFAM(tdp)
 {
 	zutp = NULL;
-  target = tdp->GetEntry();
-	mul = tdp->GetMul();
+	tdfp = tdp;
+  //target = tdp->GetEntry();
+	//mul = tdp->GetMul();
 } // end of UNZFAM standard constructor
 
 UNZFAM::UNZFAM(PUNZFAM txfp) : MAPFAM(txfp)
 {
 	zutp = txfp->zutp;
-	target = txfp->target;
-	mul = txfp->mul;
+	tdfp = txfp->tdfp;
+	//target = txfp->target;
+	//mul = txfp->mul;
 } // end of UNZFAM copy constructor
 
 /***********************************************************************/
@@ -691,7 +748,7 @@ UNZFAM::UNZFAM(PUNZFAM txfp) : MAPFAM(txfp)
 /***********************************************************************/
 int UNZFAM::GetFileLength(PGLOBAL g)
 {
-	int len = (zutp && zutp->entryopen) ? Top - Memory
+	int len = (zutp && zutp->entryopen) ? (int)(Top - Memory)
 		                                  : TXTFAM::GetFileLength(g) * 3;
 
 	if (trace)
@@ -711,7 +768,13 @@ int UNZFAM::Cardinality(PGLOBAL g)
 	int card = -1;
 	int len = GetFileLength(g);
 
-	card = (len / (int)Lrecl) * 2;           // Estimated ???
+	if (len) {
+		// Estimated ???
+		card = (len / (int)Lrecl) * 2;
+		card = card ? card : 10;				// Lrecl can be too big 
+	} else
+		card = 0;
+
 	return card;
 } // end of Cardinality
 
@@ -726,7 +789,7 @@ bool UNZFAM::OpenTableFile(PGLOBAL g)
 	/*********************************************************************/
 	/*  Allocate the ZIP utility class.                                  */
 	/*********************************************************************/
-	zutp = new(g) UNZIPUTL(target, mul);
+	zutp = new(g) UNZIPUTL(tdfp);
 
 	//  We used the file name relative to recorded datapath
 	PlugSetPath(filename, To_File, Tdbp->GetPath());
@@ -841,17 +904,19 @@ void UNZFAM::CloseTableFile(PGLOBAL g, bool)
 UZXFAM::UZXFAM(PDOSDEF tdp) : MPXFAM(tdp)
 {
 	zutp = NULL;
-	target = tdp->GetEntry();
-	mul = tdp->GetMul();
+	tdfp = tdp;
+	//target = tdp->GetEntry();
+	//mul = tdp->GetMul();
 	//Lrecl = tdp->GetLrecl();
 } // end of UZXFAM standard constructor
 
 UZXFAM::UZXFAM(PUZXFAM txfp) : MPXFAM(txfp)
 {
 	zutp = txfp->zutp;
-	target = txfp->target;
-	mul = txfp->mul;
-//Lrecl = txfp->Lrecl;
+	tdfp = txfp->tdfp;
+	//target = txfp->target;
+	//mul = txfp->mul;
+	//Lrecl = txfp->Lrecl;
 } // end of UZXFAM copy constructor
 
 /***********************************************************************/
@@ -907,7 +972,7 @@ bool UZXFAM::OpenTableFile(PGLOBAL g)
 		/*  Allocate the ZIP utility class.                                  */
 		/*********************************************************************/
 		if (!zutp)
-			zutp = new(g)UNZIPUTL(target, mul);
+			zutp = new(g)UNZIPUTL(tdfp);
 
 		//  We used the file name relative to recorded datapath
 		PlugSetPath(filename, To_File, Tdbp->GetPath());
@@ -969,6 +1034,25 @@ bool ZIPFAM::OpenTableFile(PGLOBAL g)
 {
 	char filename[_MAX_PATH];
 	MODE mode = Tdbp->GetMode();
+	int  len = TXTFAM::GetFileLength(g);
+
+	//  We used the file name relative to recorded datapath
+	PlugSetPath(filename, To_File, Tdbp->GetPath());
+
+	if (len < 0)
+		return true;
+	else if (!append && len > 0) {
+		strcpy(g->Message, "No insert into existing zip file");
+		return true;
+	} else if (append && len > 0) {
+		UNZIPUTL *zutp = new(g) UNZIPUTL(target, false);
+
+		if (!zutp->IsInsertOk(g, filename)) {
+			strcpy(g->Message, "No insert into existing entry");
+			return true;
+		}	// endif Ok
+
+	} // endif's
 
 	/*********************************************************************/
 	/*  Allocate the ZIP utility class.                                  */
@@ -1004,7 +1088,7 @@ int ZIPFAM::WriteBuffer(PGLOBAL g)
 
 	//  Prepare to write the new line
 	strcat(strcpy(To_Buf, Tdbp->GetLine()), (Bin) ? CrLf : "\n");
-	len = strchr(To_Buf, '\n') - To_Buf + 1;
+	len = (int)(strchr(To_Buf, '\n') - To_Buf + 1);
 	return zutp->writeEntry(g, To_Buf, len);
 } // end of WriteBuffer
 
@@ -1028,15 +1112,31 @@ ZPXFAM::ZPXFAM(PDOSDEF tdp) : FIXFAM(tdp)
 	target = tdp->GetEntry();
 	append = tdp->GetAppend();
 	//Lrecl = tdp->GetLrecl();
-} // end of UZXFAM standard constructor
+} // end of ZPXFAM standard constructor
 
 /***********************************************************************/
 /*  OpenTableFile: Open a DOS/UNIX table file from a ZIP file.         */
 /***********************************************************************/
 bool ZPXFAM::OpenTableFile(PGLOBAL g)
 {
-	char    filename[_MAX_PATH];
-	MODE    mode = Tdbp->GetMode();
+	char filename[_MAX_PATH];
+	MODE mode = Tdbp->GetMode();
+	int  len = TXTFAM::GetFileLength(g);
+
+	if (len < 0)
+		return true;
+	else if (!append && len > 0) {
+		strcpy(g->Message, "No insert into existing zip file");
+		return true;
+	} else if (append && len > 0) {
+		UNZIPUTL *zutp = new(g) UNZIPUTL(target, false);
+
+		if (!zutp->IsInsertOk(g, filename)) {
+			strcpy(g->Message, "No insert into existing entry");
+			return true;
+		}	// endif Ok
+
+	} // endif's
 
 	/*********************************************************************/
 	/*  Allocate the ZIP utility class.                                  */
