@@ -124,8 +124,8 @@ bool MYSQLDEF::GetServerInfo(PGLOBAL g, const char *server_name)
     DBUG_RETURN(true);
     } // endif server
 
-  DBUG_PRINT("info", ("get_server_by_name returned server at %zx",
-                      (size_t) server));
+  DBUG_PRINT("info", ("get_server_by_name returned server at %p",
+                     server));
 
   // TODO: We need to examine which of these can really be NULL
   Hostname = PlugDup(g, server->host);
@@ -513,18 +513,8 @@ bool TDBMYSQL::MakeSelect(PGLOBAL g, bool mx)
   if (Query)
     return false;        // already done
 
-	if (Srcdef) {
-		if (strstr(Srcdef, "%s")) {
-			char *fil;
-
-			fil = (To_CondFil) ? To_CondFil->Body : PlugDup(g, "1=1");
-			Query = new(g)STRING(g, strlen(Srcdef) + strlen(fil));
-			Query->SetLength(sprintf(Query->GetStr(), Srcdef, fil));
-		} else
-			Query = new(g)STRING(g, 0, Srcdef);
-
-		return false;
-	} // endif Srcdef
+	if (Srcdef)
+		return MakeSrcdef(g);
 
   // Allocate the string used to contain Query
   Query = new(g) STRING(g, 1023, "SELECT ");
@@ -691,7 +681,7 @@ bool TDBMYSQL::MakeCommand(PGLOBAL g)
       strlwr(strcpy(name, Name));     // Not a keyword
 
     if ((p = strstr(qrystr, name))) {
-      Query->Set(Qrystr, p - qrystr);
+      Query->Set(Qrystr, (uint)(p - qrystr));
 
       if (qtd && *(p-1) == ' ') {
         Query->Append('`');
@@ -1270,7 +1260,8 @@ MYSQLCOL::MYSQLCOL(MYSQL_FIELD *fld, PTDB tdbp, int i, PCSZ am)
         : COLBLK(NULL, tdbp, i)
   {
   const char *chset = get_charset_name(fld->charsetnr);
-  char  v = (!strcmp(chset, "binary")) ? 'B' : 0;
+//char  v = (!strcmp(chset, "binary")) ? 'B' : 0;
+	char  v = 0;
 
   Name = fld->name;
   Opt = 0;
