@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2016, MariaDB Corporation. All Rights Reserved.
+Copyright (c) 2016, 2017, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -29,6 +29,7 @@ Created 25/08/2016 Jan Lindström
 #include "dict0defrag_bg.h"
 #include "row0mysql.h"
 #include "srv0start.h"
+#include "trx0roll.h"
 #include "ut0new.h"
 
 #include <vector>
@@ -201,19 +202,18 @@ dict_stats_defrag_pool_del(
 	mutex_exit(&defrag_pool_mutex);
 }
 
-/*****************************************************************//**
-Get the first index that has been added for updating persistent defrag
-stats and eventually save its stats. */
+/** Get the first index that has been added for updating persistent defrag
+stats and eventually save its stats.
+@param[in,out]	trx	transaction that will be started and committed  */
 static
 void
-dict_stats_process_entry_from_defrag_pool()
-/*=======================================*/
+dict_stats_process_entry_from_defrag_pool(trx_t* trx)
 {
 	table_id_t	table_id;
 	index_id_t	index_id;
-	dberr_t		err = DB_SUCCESS;
 
 	ut_ad(!srv_read_only_mode);
+	ut_ad(trx->persistent_stats);
 
 	/* pop the first index from the auto defrag pool */
 	if (!dict_stats_defrag_pool_get(&table_id, &index_id)) {
@@ -230,95 +230,76 @@ dict_stats_process_entry_from_defrag_pool()
 	table = dict_table_open_on_id(table_id, TRUE,
 				      DICT_TABLE_OP_OPEN_ONLY_IF_CACHED);
 
-	if (table == NULL) {
+	dict_index_t* index = table && !table->corrupted
+		? dict_table_find_index_on_id(table, index_id)
+		: NULL;
+
+	if (!index || dict_index_is_corrupted(index)) {
+		if (table) {
+			dict_table_close(table, TRUE, FALSE);
+		}
 		mutex_exit(&dict_sys->mutex);
 		return;
 	}
 
-	/* Check whether table is corrupted */
-	if (table->corrupted) {
-		dict_table_close(table, TRUE, FALSE);
-		mutex_exit(&dict_sys->mutex);
-		return;
-	}
 	mutex_exit(&dict_sys->mutex);
-
-	dict_index_t*	index = dict_table_find_index_on_id(table, index_id);
-
-	if (index == NULL) {
-		return;
-	}
-
-	/* Check whether index is corrupted */
-	if (dict_index_is_corrupted(index)) {
-		dict_table_close(table, FALSE, FALSE);
-		return;
-	}
-
-	err = dict_stats_save_defrag_stats(index);
+	trx->error_state = DB_SUCCESS;
+	++trx->will_lock;
+	dberr_t err = dict_stats_save_defrag_stats(index, trx);
 
 	if (err != DB_SUCCESS) {
+		trx_rollback_to_savepoint(trx, NULL);
 		ib::error() << "Saving defragmentation status for table "
-			    << index->table->name.m_name
-			    << " index " << index->name()
+			    << index->table->name
+			    << " index " << index->name
 			    << " failed " << err;
+	} else if (trx->state != TRX_STATE_NOT_STARTED) {
+		trx_commit_for_mysql(trx);
 	}
 
 	dict_table_close(table, FALSE, FALSE);
 }
 
-/*****************************************************************//**
-Get the first index that has been added for updating persistent defrag
-stats and eventually save its stats. */
+/** Process indexes that have been scheduled for defragmenting.
+@param[in,out]	trx	transaction that will be started and committed  */
 void
-dict_defrag_process_entries_from_defrag_pool()
-/*==========================================*/
+dict_defrag_process_entries_from_defrag_pool(trx_t* trx)
 {
 	while (defrag_pool->size() && !dict_stats_start_shutdown) {
-		dict_stats_process_entry_from_defrag_pool();
+		dict_stats_process_entry_from_defrag_pool(trx);
 	}
 }
 
-/*********************************************************************//**
-Save defragmentation result.
+/** Save defragmentation result.
+@param[in]	index	index that was defragmented
+@param[in,out]	trx	transaction
 @return DB_SUCCESS or error code */
 dberr_t
-dict_stats_save_defrag_summary(
-/*============================*/
-	dict_index_t*	index)	/*!< in: index */
+dict_stats_save_defrag_summary(dict_index_t* index, trx_t* trx)
 {
-	dberr_t	ret=DB_SUCCESS;
-	lint	now = (lint) ut_time();
+	ut_ad(trx->persistent_stats);
 
 	if (dict_index_is_ibuf(index)) {
 		return DB_SUCCESS;
 	}
 
-	rw_lock_x_lock(dict_operation_lock);
-	mutex_enter(&dict_sys->mutex);
-
-	ret = dict_stats_save_index_stat(index, now, "n_pages_freed",
-					 index->stat_defrag_n_pages_freed,
-					 NULL,
-					 "Number of pages freed during"
-					 " last defragmentation run.",
-					 NULL);
-
-	mutex_exit(&dict_sys->mutex);
-	rw_lock_x_unlock(dict_operation_lock);
-
-	return (ret);
+	return dict_stats_save_index_stat(index, ut_time(), "n_pages_freed",
+					  index->stat_defrag_n_pages_freed,
+					  NULL,
+					  "Number of pages freed during"
+					  " last defragmentation run.",
+					  trx);
 }
 
-/*********************************************************************//**
-Save defragmentation stats for a given index.
+/** Save defragmentation stats for a given index.
+@param[in]	index	index that is being defragmented
+@param[in,out]	trx	transaction
 @return DB_SUCCESS or error code */
 dberr_t
-dict_stats_save_defrag_stats(
-/*============================*/
-	dict_index_t*	index)	/*!< in: index */
+dict_stats_save_defrag_stats(dict_index_t* index, trx_t* trx)
 {
-	dberr_t	ret;
+	ut_ad(trx->error_state == DB_SUCCESS);
+	ut_ad(trx->persistent_stats);
 
 	if (dict_index_is_ibuf(index)) {
 		return DB_SUCCESS;
@@ -328,7 +309,6 @@ dict_stats_save_defrag_stats(
 		return dict_stats_report_error(index->table, true);
 	}
 
-	lint	now = (lint) ut_time();
 	mtr_t	mtr;
 	ulint	n_leaf_pages;
 	ulint	n_leaf_reserved;
@@ -345,40 +325,33 @@ dict_stats_save_defrag_stats(
 		return DB_SUCCESS;
 	}
 
-	rw_lock_x_lock(dict_operation_lock);
-
-	mutex_enter(&dict_sys->mutex);
-	ret = dict_stats_save_index_stat(index, now, "n_page_split",
-					 index->stat_defrag_n_page_split,
-					 NULL,
-					 "Number of new page splits on leaves"
-					 " since last defragmentation.",
-					 NULL);
-	if (ret != DB_SUCCESS) {
-		goto end;
+	ib_time_t now = ut_time();
+	dberr_t err = dict_stats_save_index_stat(
+		index, now, "n_page_split",
+		index->stat_defrag_n_page_split,
+		NULL,
+		"Number of new page splits on leaves"
+		" since last defragmentation.",
+		trx);
+	if (err == DB_SUCCESS) {
+		err = dict_stats_save_index_stat(
+			index, now, "n_leaf_pages_defrag",
+			n_leaf_pages,
+			NULL,
+			"Number of leaf pages when this stat is saved to disk",
+			trx);
 	}
 
-	ret = dict_stats_save_index_stat(
-		index, now, "n_leaf_pages_defrag",
-		n_leaf_pages,
-		NULL,
-		"Number of leaf pages when this stat is saved to disk",
-		NULL);
-	if (ret != DB_SUCCESS) {
-		goto end;
+	if (err == DB_SUCCESS) {
+		err = dict_stats_save_index_stat(
+			index, now, "n_leaf_pages_reserved",
+			n_leaf_reserved,
+			NULL,
+			"Number of pages reserved for this "
+			"index leaves when this stat "
+			"is saved to disk",
+			trx);
 	}
 
-	ret = dict_stats_save_index_stat(
-		index, now, "n_leaf_pages_reserved",
-		n_leaf_reserved,
-		NULL,
-		"Number of pages reserved for this index leaves when this stat "
-		"is saved to disk",
-		NULL);
-
-end:
-	mutex_exit(&dict_sys->mutex);
-	rw_lock_x_unlock(dict_operation_lock);
-
-	return (ret);
+	return err;
 }
