@@ -198,13 +198,6 @@ ha_validate(
 	ulint		start_index,	/*!< in: start index */
 	ulint		end_index);	/*!< in: end index */
 #endif /* defined UNIV_AHI_DEBUG || defined UNIV_DEBUG */
-/*************************************************************//**
-Prints info of a hash table. */
-void
-ha_print_info(
-/*==========*/
-	FILE*		file,	/*!< in: file where to print */
-	hash_table_t*	table);	/*!< in: hash table */
 
 /** The hash table external chain node */
 struct ha_node_t {
