@@ -190,7 +190,7 @@ lock_update_merge_left(
 	const buf_block_t*	right_block);	/*!< in: merged index page
 						which will be discarded */
 /*************************************************************//**
-Updates the lock table when a page is splited and merged to
+Updates the lock table when a page is split and merged to
 two pages. */
 UNIV_INTERN
 void
@@ -1073,16 +1073,9 @@ std::string
 lock_get_info(
 	const lock_t*);
 
-/*************************************************************//**
-Updates the lock table when a page is split and merged to
-two pages. */
-UNIV_INTERN
-void
-lock_update_split_and_merge(
-	const buf_block_t* left_block,	/*!< in: left page to which merged */
-	const rec_t* orig_pred,		/*!< in: original predecessor of
-					supremum on the left page before merge*/
-	const buf_block_t* right_block);/*!< in: right page from which merged */
+/*******************************************************************//**
+@return whether wsrep_on is true on trx->mysql_thd*/
+#define wsrep_on_trx(trx) ((trx)->mysql_thd && wsrep_on((trx)->mysql_thd))
 
 #endif /* WITH_WSREP */
 
