@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All rights reserved.
 Copyright (c) 2009, Google Inc.
-Copyright (c) 2017, MariaDB Corporation. All Rights Reserved.
+Copyright (c) 2017, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -39,6 +39,7 @@ Created 12/9/1995 Heikki Tuuri
 #include "sync0rw.h"
 #include "log0types.h"
 #include "os0event.h"
+#include "os0file.h"
 
 /** Redo log group */
 struct log_group_t;
@@ -151,24 +152,24 @@ UNIV_INLINE
 lsn_t
 log_get_max_modified_age_async(void);
 /*================================*/
-/******************************************************//**
-Initializes the log. */
+/** Initializes the redo logging subsystem. */
 void
-log_init(void);
-/*==========*/
-/******************************************************************//**
-Inits a log group to the log system.
-@return true if success, false if not */
-MY_ATTRIBUTE((warn_unused_result))
+log_sys_init();
+
+/** Initialize the redo log.
+@param[in]	n_files		number of files */
+void
+log_init(ulint n_files);
+/** Calculate the recommended highest values for lsn - last_checkpoint_lsn
+and lsn - buf_get_oldest_modification().
+@param[in]	file_size	requested innodb_log_file_size
+@retval true on success
+@retval false if the smallest log group is too small to
+accommodate the number of OS threads in the database server */
 bool
-log_group_init(
-/*===========*/
-	ulint	id,			/*!< in: group id */
-	ulint	n_files,		/*!< in: number of log files */
-	lsn_t	file_size,		/*!< in: log file size in bytes */
-	ulint	space_id);		/*!< in: space id of the file space
-					which contains the log files of this
-					group */
+log_set_capacity(ulonglong file_size)
+	MY_ATTRIBUTE((warn_unused_result));
+
 /******************************************************//**
 Completes an i/o to a log file. */
 void
@@ -407,16 +408,9 @@ Closes all log groups. */
 void
 log_group_close_all(void);
 /*=====================*/
-/********************************************************//**
-Shutdown the log system but do not release all the memory. */
+/** Shut down the redo log subsystem. */
 void
-log_shutdown(void);
-/*==============*/
-/********************************************************//**
-Free the log system data structures. */
-void
-log_mem_free(void);
-/*==============*/
+log_shutdown();
 
 /** Whether to generate and require checksums on the redo log pages */
 extern my_bool	innodb_log_checksums;
@@ -552,16 +546,12 @@ Currently, this is only protected by log_sys->mutex. However, in the case
 of log_write_up_to(), we will access some members only with the protection
 of log_sys->write_mutex, which should affect nothing for now. */
 struct log_group_t{
-	/** log group identifier (always 0) */
-	ulint				id;
 	/** number of files in the group */
 	ulint				n_files;
 	/** format of the redo log: e.g., LOG_HEADER_FORMAT_CURRENT */
 	ulint				format;
 	/** individual log file size in bytes, including the header */
-	lsn_t				file_size
-	/** file space which implements the log group */;
-	ulint				space_id;
+	lsn_t				file_size;
 	/** corruption status */
 	log_group_state_t		state;
 	/** lsn used to fix coordinates within the log group */
@@ -580,14 +570,18 @@ struct log_group_t{
 	byte*				checkpoint_buf_ptr;
 	/** buffer for writing a checkpoint header */
 	byte*				checkpoint_buf;
-	/** list of log groups */
-	UT_LIST_NODE_T(log_group_t)	log_groups;
 
 	/** @return whether the redo log is encrypted */
 	bool is_encrypted() const
 	{
 		return((format & LOG_HEADER_FORMAT_ENCRYPTED) != 0);
 	}
+
+	/** @return capacity in bytes */
+	inline lsn_t capacity() const
+	{
+		return((file_size - LOG_FILE_HDR_SIZE) * n_files);
+	}
 };
 
 /** Redo log buffer */
@@ -639,8 +633,8 @@ struct log_t{
 					max_checkpoint_age; this flag is
 					peeked at by log_free_check(), which
 					does not reserve the log mutex */
-	UT_LIST_BASE_NODE_T(log_group_t)
-			log_groups;	/*!< log groups */
+	/** the redo log */
+	log_group_t			log;
 
 	/** The fields involved in the log buffer flush @{ */
 
@@ -729,7 +723,7 @@ struct log_t{
 	/** @return whether the redo log is encrypted */
 	bool is_encrypted() const
 	{
-		return(UT_LIST_GET_FIRST(log_groups)->is_encrypted());
+		return(log.is_encrypted());
 	}
 };
 
