@@ -260,9 +260,9 @@ enum latch_level_t {
 	SYNC_TREE_NODE,
 	SYNC_TREE_NODE_FROM_HASH,
 	SYNC_TREE_NODE_NEW,
+	SYNC_IBUF_PESS_INSERT_MUTEX,
 	SYNC_INDEX_TREE,
 
-	SYNC_IBUF_PESS_INSERT_MUTEX,
 	SYNC_IBUF_HEADER,
 	SYNC_DICT_HEADER,
 	SYNC_STATS_AUTO_RECALC,
@@ -270,10 +270,10 @@ enum latch_level_t {
 	SYNC_DICT,
 	SYNC_FTS_CACHE,
 
-	SYNC_DICT_OPERATION,
-
 	SYNC_FILE_FORMAT_TAG,
 
+	SYNC_DICT_OPERATION,
+
 	SYNC_TRX_I_S_LAST_READ,
 
 	SYNC_TRX_I_S_RWLOCK,
@@ -335,7 +335,6 @@ enum latch_id_t {
 	LATCH_ID_RTR_PATH_MUTEX,
 	LATCH_ID_RW_LOCK_LIST,
 	LATCH_ID_RW_LOCK_MUTEX,
-	LATCH_ID_SRV_DICT_TMPFILE,
 	LATCH_ID_SRV_INNODB_MONITOR,
 	LATCH_ID_SRV_MISC_TMPFILE,
 	LATCH_ID_SRV_MONITOR_FILE,
@@ -1193,10 +1192,16 @@ struct MY_ALIGNED(CPU_LEVEL1_DCACHE_LINESIZE) simple_counter
 	{
 		compile_time_assert(!atomic || sizeof(Type) == sizeof(lint));
 		if (atomic) {
-			/* Silence MSVS warnings when instantiating
-			this template with atomic=false. */
+#ifdef _MSC_VER
+// Suppress type conversion/ possible loss of data warning
+#pragma warning (push)
+#pragma warning (disable : 4244)
+#endif
 			return Type(my_atomic_addlint(reinterpret_cast<lint*>
 						      (&m_counter), i));
+#ifdef _MSC_VER
+#pragma warning (pop)
+#endif
 		} else {
 			return m_counter += i;
 		}
