@@ -110,9 +110,12 @@ void
 trx_rseg_mem_free(trx_rseg_t* rseg);
 
 /** Create a persistent rollback segment.
-@param[in]	space_id	system or undo tablespace id */
+@param[in]	space_id	system or undo tablespace id
+@return pointer to new rollback segment
+@retval	NULL	on failure */
 trx_rseg_t*
-trx_rseg_create(ulint space_id);
+trx_rseg_create(ulint space_id)
+	MY_ATTRIBUTE((warn_unused_result));
 
 /** Create the temporary rollback segments. */
 void
@@ -200,9 +203,17 @@ struct trx_rseg_t {
 	bool is_persistent() const
 	{
 		ut_ad(space == SRV_TMP_SPACE_ID
-		      || space <= TRX_SYS_MAX_UNDO_SPACES);
+		      || space == TRX_SYS_SPACE
+		      || (srv_undo_space_id_start > 0
+			  && space >= srv_undo_space_id_start
+			  && space <= srv_undo_space_id_start
+			  + TRX_SYS_MAX_UNDO_SPACES));
 		ut_ad(space == SRV_TMP_SPACE_ID
-		      || space <= srv_undo_tablespaces_active
+		      || space == TRX_SYS_SPACE
+		      || (srv_undo_space_id_start > 0
+			  && space >= srv_undo_space_id_start
+			  && space <= srv_undo_space_id_start
+			  + srv_undo_tablespaces_active)
 		      || !srv_was_started);
 		return(space != SRV_TMP_SPACE_ID);
 	}
