@@ -417,7 +417,8 @@ que_graph_free_recursive(
 	}
 
 	DBUG_PRINT("que_graph_free_recursive",
-		   ("node: %p, type: %lu", node, que_node_get_type(node)));
+		   ("node: %p, type: " ULINTPF, node,
+		    que_node_get_type(node)));
 
 	switch (que_node_get_type(node)) {
 
