@@ -1271,10 +1271,6 @@ row_truncate_complete(
 		}
 	}
 
-	if (err == DB_SUCCESS) {
-		dict_stats_update(table, DICT_STATS_EMPTY_TABLE);
-	}
-
 	trx->op_info = "";
 
 	/* For temporary tables or if there was an error, we need to reset
@@ -2107,8 +2103,17 @@ row_truncate_table_for_mysql(
 	dict_table_autoinc_unlock(table);
 
 	if (trx_is_started(trx)) {
+		char	errstr[1024];
+		if (dict_stats_drop_table(table->name.m_name, errstr,
+					  sizeof errstr, trx) != DB_SUCCESS) {
+			ib::warn() << "Deleting persistent "
+				"statistics for table " << table->name
+				   << " failed: " << errstr;
+		}
 
 		trx_commit_for_mysql(trx);
+
+		dict_stats_empty_table(table);
 	}
 
 	return(row_truncate_complete(table, trx, fsp_flags, logger, err));
