@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /* Create a MyISAM table */
 
@@ -274,7 +274,7 @@ int mi_create(const char *name,uint keys,MI_KEYDEF *keydefs,
            so we only need to decrease keydef->keysegs.
            (see recreate_table() in mi_check.c)
         */
-        keydef->keysegs-=sp_segs-1;
+        keydef->keysegs= 1;
       }
 
       for (j=0, keyseg=keydef->seg ; (int) j < keydef->keysegs ;
@@ -288,7 +288,8 @@ int mi_create(const char *name,uint keys,MI_KEYDEF *keydefs,
           goto err_no_lock;
         }
       }
-      keydef->keysegs+=sp_segs;
+      DBUG_ASSERT(keydef->keysegs == 1);
+      keydef->keysegs= sp_segs + 1;
       key_length+=SPLEN*sp_segs;
       length++;                              /* At least one length byte */
       min_key_length_skip+=SPLEN*2*SPDIMS;
@@ -610,7 +611,7 @@ int mi_create(const char *name,uint keys,MI_KEYDEF *keydefs,
     klinkname_ptr= klinkname;
     /*
       Don't create the table if the link or file exists to ensure that one
-      doesn't accidentally destroy another table.
+      doesn't accidently destroy another table.
     */
     create_flag=0;
   }
