@@ -7,6 +7,7 @@
  * See COPYRIGHT.txt for details.
  */
 
+#include <my_global.h>
 #include "mysql_version.h"
 #if MYSQL_VERSION_ID < 50500
 #include "mysql_priv.h"
@@ -262,8 +263,8 @@ parse_args(int argc, char **argv, config& conf)
     }
     if (!(param = new conf_param()))
       continue;
-    uint32 key_len = eq - arg;
-    uint32 val_len = strlen(eq + 1);
+    uint32 key_len = (uint32)(eq - arg);
+    uint32 val_len = (uint32)(strlen(eq + 1));
     if (
       param->key.reserve(key_len + 1) ||
       param->val.reserve(val_len + 1)
