@@ -11,7 +11,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
 
 #define SPIDER_HS_CONN dena::hstcpcli_ptr
 #define SPIDER_HS_CONN_CREATE dena::hstcpcli_i::create
@@ -776,6 +776,9 @@ public:
     ulong sql_type,
     int link_idx
   );
+  bool is_sole_projection_field(
+    uint16 field_index
+  );
   bool is_bulk_insert_exec_period(
     bool bulk_end
   );
