@@ -36,6 +36,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 
 #ident "Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved."
 
+#include <my_global.h>
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
@@ -48,10 +49,6 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #include "loader/dbufio.h"
 #include "loader/loader-internal.h"
 
-toku_instr_key *bfs_mutex_key;
-toku_instr_key *bfs_cond_key;
-toku_instr_key *io_thread_key;
-
 struct dbufio_file {
     // i/o thread owns these
     int fd;
@@ -139,7 +136,7 @@ static ssize_t dbf_read_some_compressed(struct dbufio_file *dbf, char *buf, size
         ret = 0;
         goto exit;
     }
-    if (readcode < header_size) {
+    if (readcode < (ssize_t) header_size) {
         errno = TOKUDB_NO_DATA;
         ret = -1;
         goto exit;
@@ -167,7 +164,7 @@ static ssize_t dbf_read_some_compressed(struct dbufio_file *dbf, char *buf, size
         ret = -1;
         goto exit;
     }
-    if (readcode < total_size) {
+    if (readcode < (ssize_t) total_size) {
         errno = TOKUDB_NO_DATA;
         ret = -1;
         goto exit;
@@ -279,44 +276,39 @@ static void* io_thread (void *v)
     toku_mutex_lock(&bfs->mutex);
     //printf("%s:%d Locked\n", __FILE__, __LINE__);
     while (1) {
-        if (paniced(bfs)) {
-            toku_mutex_unlock(&bfs->mutex);  // ignore any error
-            toku_instr_delete_current_thread();
-            return toku_pthread_done(nullptr);
-        }
-        // printf("n_not_done=%d\n", bfs->n_not_done);
-        if (bfs->n_not_done == 0) {
-            // all done (meaning we stored EOF (or another error) in
-            // error_code[0] for the file.
-            // printf("unlocked\n");
-            toku_mutex_unlock(&bfs->mutex);
-            toku_instr_delete_current_thread();
-            return toku_pthread_done(nullptr);
-        }
 
-        struct dbufio_file *dbf = bfs->head;
-        if (dbf == NULL) {
-            // No I/O needs to be done yet.
-            // Wait until something happens that will wake us up.
-            toku_cond_wait(&bfs->cond, &bfs->mutex);
-            if (paniced(bfs)) {
-                toku_mutex_unlock(&bfs->mutex);  // ignore any error
-                toku_instr_delete_current_thread();
-                return toku_pthread_done(nullptr);
-            }
-            // Have the lock so go around.
-        } else {
-            // Some I/O needs to be done.
-            // printf("%s:%d Need I/O\n", __FILE__, __LINE__);
-            assert(dbf->second_buf_ready == false);
-            assert(!dbf->io_done);
-            bfs->head = dbf->next;
-            if (bfs->head == NULL)
-                bfs->tail = NULL;
+	if (paniced(bfs)) {
+	    toku_mutex_unlock(&bfs->mutex); // ignore any error
+	    return 0;
+	}
+	//printf("n_not_done=%d\n", bfs->n_not_done);
+	if (bfs->n_not_done==0) {
+	    // all done (meaning we stored EOF (or another error) in error_code[0] for the file.
+	    //printf("unlocked\n");
+	    toku_mutex_unlock(&bfs->mutex);
+	    return 0;
+	}
 
-            // Unlock the mutex now that we have ownership of dbf to allow
-            // consumers to get the mutex and perform swaps.  They won't swap
-            // this buffer because second_buf_ready is false.
+	struct dbufio_file *dbf = bfs->head;
+	if (dbf==NULL) {
+	    // No I/O needs to be done yet. 
+	    // Wait until something happens that will wake us up.
+	    toku_cond_wait(&bfs->cond, &bfs->mutex);
+	    if (paniced(bfs)) {
+		toku_mutex_unlock(&bfs->mutex); // ignore any error
+		return 0;
+	    }
+	    // Have the lock so go around.
+	} else {
+	    // Some I/O needs to be done.
+	    //printf("%s:%d Need I/O\n", __FILE__, __LINE__);
+	    assert(dbf->second_buf_ready == false);
+	    assert(!dbf->io_done);
+	    bfs->head = dbf->next;
+	    if (bfs->head==NULL) bfs->tail=NULL;
+
+	    // Unlock the mutex now that we have ownership of dbf to allow consumers to get the mutex and perform swaps.  They won't swap
+	    // this buffer because second_buf_ready is false.
             toku_mutex_unlock(&bfs->mutex);
 	    //printf("%s:%d Doing read fd=%d\n", __FILE__, __LINE__, dbf->fd);
 	    {
@@ -347,16 +339,14 @@ static void* io_thread (void *v)
 
 		//printf("%s:%d locking mutex again=%ld\n", __FILE__, __LINE__, readcode);
 		{
-            toku_mutex_lock(&bfs->mutex);
-            if (paniced(bfs)) {
-                toku_mutex_unlock(&bfs->mutex);  // ignore any error
-                toku_instr_delete_current_thread();
-                return toku_pthread_done(nullptr);
-            }
-        }
-        // Now that we have the mutex, we can decrement n_not_done (if
-        // applicable) and set second_buf_ready
-        if (readcode<=0) {
+		    toku_mutex_lock(&bfs->mutex);
+		    if (paniced(bfs)) {
+                        toku_mutex_unlock(&bfs->mutex); // ignore any error
+                        return 0;
+                    }
+		}
+		// Now that we have the mutex, we can decrement n_not_done (if applicable) and set second_buf_ready
+		if (readcode<=0) {
 		    bfs->n_not_done--;
 		}
 		//printf("%s:%d n_not_done=%d\n", __FILE__, __LINE__, bfs->n_not_done);
@@ -387,36 +377,34 @@ int create_dbufio_fileset (DBUFIO_FILESET *bfsp, int N, int fds[/*N*/], size_t b
 	    }
 	}
     }
-    // printf("%s:%d here\n", __FILE__, __LINE__);
-    if (result == 0) {
-        toku_mutex_init(*bfs_mutex_key, &bfs->mutex, nullptr);
-        mutex_inited = true;
+    //printf("%s:%d here\n", __FILE__, __LINE__);
+    if (result==0) {
+	toku_mutex_init(&bfs->mutex, NULL);
+	mutex_inited = true;
     }
-    if (result == 0) {
-        toku_cond_init(*bfs_cond_key, &bfs->cond, nullptr);
-        cond_inited = true;
+    if (result==0) {
+	toku_cond_init(&bfs->cond, NULL);
+	cond_inited = true;
     }
-    if (result == 0) {
-        bfs->N = N;
-        bfs->n_not_done = N;
-        bfs->head = bfs->tail = NULL;
-        for (int i = 0; i < N; i++) {
-            bfs->files[i].fd = fds[i];
-            bfs->files[i].offset_in_buf = 0;
-            bfs->files[i].offset_in_uncompressed_file = 0;
-            bfs->files[i].next = NULL;
-            bfs->files[i].second_buf_ready = false;
-            for (int j = 0; j < 2; j++) {
-                if (result == 0) {
-                    MALLOC_N(bufsize, bfs->files[i].buf[j]);
-                    if (bfs->files[i].buf[j] == NULL) {
-                        result = get_error_errno();
-                    }
-                }
-                bfs->files[i].n_in_buf[j] = 0;
-                bfs->files[i].error_code[j] = 0;
-            }
-            bfs->files[i].io_done = false;
+    if (result==0) {
+	bfs->N = N;
+	bfs->n_not_done = N;
+	bfs->head = bfs->tail = NULL;
+	for (int i=0; i<N; i++) {
+	    bfs->files[i].fd = fds[i];
+	    bfs->files[i].offset_in_buf = 0;
+	    bfs->files[i].offset_in_uncompressed_file = 0;
+	    bfs->files[i].next = NULL;
+	    bfs->files[i].second_buf_ready = false;
+	    for (int j=0; j<2; j++) {
+		if (result==0) {
+		    MALLOC_N(bufsize, bfs->files[i].buf[j]);
+		    if (bfs->files[i].buf[j]==NULL) { result=get_error_errno(); }
+		}
+		bfs->files[i].n_in_buf[j] = 0;
+		bfs->files[i].error_code[j] = 0;
+	    }
+	    bfs->files[i].io_done = false;
             ssize_t r;
             if (bfs->compressed) {
                 r = dbf_read_compressed(&bfs->files[i], bfs->files[i].buf[0], bufsize);
@@ -443,21 +431,13 @@ int create_dbufio_fileset (DBUFIO_FILESET *bfsp, int N, int fds[/*N*/], size_t b
 	bfs->panic = false;
 	bfs->panic_errno = 0;
     }
-    // printf("Creating IO thread\n");
-    if (result == 0) {
-        result = toku_pthread_create(*io_thread_key,
-                                     &bfs->iothread,
-                                     nullptr,
-                                     io_thread,
-                                     static_cast<void *>(bfs));
-    }
-    if (result == 0) {
-        *bfsp = bfs;
-        return 0;
+    //printf("Creating IO thread\n");
+    if (result==0) {
+	result = toku_pthread_create(&bfs->iothread, NULL, io_thread, (void*)bfs);
     }
+    if (result==0) { *bfsp = bfs; return 0; }
     // Now undo everything.
-    // If we got here, there is no thread (either result was zero before the
-    // thread was created, or else the thread creation itself failed.
+    // If we got here, there is no thread (either result was zero before the thread was created, or else the thread creation itself failed.
     if (bfs) {
 	if (bfs->files) {
 	    // the files were allocated, so we have to free all the bufs.
@@ -586,7 +566,7 @@ dbufio_print(DBUFIO_FILESET bfs) {
     fprintf(stderr, "%s:%d bfs=%p", __FILE__, __LINE__, bfs);
     if (bfs->panic)
         fprintf(stderr, " panic=%d", bfs->panic_errno);
-    fprintf(stderr, " N=%d %d %" PRIuMAX, bfs->N, bfs->n_not_done, bfs->bufsize);
+    fprintf(stderr, " N=%d %d %" PRIuMAX, bfs->N, bfs->n_not_done, (uintmax_t) bfs->bufsize);
     for (int i = 0; i < bfs->N; i++) {
         struct dbufio_file *dbf = &bfs->files[i];
         if (dbf->error_code[0] || dbf->error_code[1])
