@@ -1,7 +1,6 @@
 /*****************************************************************************
 
 Copyright (c) 2011-2012, Percona Inc. All Rights Reserved.
-Copyright (c) 2019, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -130,11 +129,7 @@ log_online_bitmap_iterator_next(
 
 /** Struct for single bitmap file information */
 struct log_online_bitmap_file_struct {
-	/** Name with full path
-	    61 is a nice magic constant for the extra space needed for the sprintf
-	    template in the cc file
-	*/
-	char		name[FN_REFLEN+61];	/*!< Name with full path */
+	char		name[FN_REFLEN];	/*!< Name with full path */
 	pfs_os_file_t	file;			/*!< Handle to opened file */
 	ib_uint64_t	size;			/*!< Size of the file */
 	os_offset_t	offset;			/*!< Offset of the next read,
@@ -147,7 +142,7 @@ struct log_online_bitmap_file_range_struct {
 	size_t	count;					/*!< Number of files */
 	/*!< Dynamically-allocated array of info about individual files */
 	struct files_t {
-		char	name[OS_FILE_MAX_PATH+1];/*!< Name of a file */
+		char	name[FN_REFLEN];	/*!< Name of a file */
 		lsn_t	start_lsn;		/*!< Starting LSN of data in
 						this file */
 		ulong	seq_num;		/*!< Sequence number of	this
