@@ -1051,7 +1051,7 @@ static void
 my_fill_mb2(CHARSET_INFO *cs, char *s, size_t slen, int fill)
 {
   char buf[10], *last;
-  int buflen, remainder;
+  size_t buflen, remainder;
 
   DBUG_ASSERT((slen % 2) == 0);
 
@@ -1066,7 +1066,7 @@ my_fill_mb2(CHARSET_INFO *cs, char *s, size_t slen, int fill)
   for (last= s + slen - buflen; s <= last; s+= buflen)
   {
     /* Enough space for the characer */
-    memcpy(s, buf, (size_t) buflen);
+    memcpy(s, buf, buflen);
   }
 
   /* 
